### ===========================================================================
###
### Compute calving weights.
###
### ===========================================================================
##
##  Warning, to install, configure, run, use any of Olivier Marti's
##  software or to read the associated documentation you'll need at least
##  one (1) brain in a reasonably working order. Lack of this implement
##  will void any warranties (either express or implied).
##  O. Marti assumes no responsability for errors, omissions,
##  data loss, or any other consequences caused directly or indirectly by
##  the usage of his software by incorrectly or partially configured
##  personal.
##
import numpy as np, xarray as xr
import sys, os, platform, argparse, textwrap, time
from scipy import ndimage
import nemo

## SVN information
__Author__   = "$Author$"
__Date__     = "$Date$"
__Revision__ = "$Revision$"
__Id__       = "$Id$"
__HeadURL__  = "$HeadURL$"
__SVN_Date__ = "$SVN_Date: $"

###

### ===== Handling command line parameters ==================================================
# Creating a parser
parser = argparse.ArgumentParser (
    description = """Compute calving weights""",
    epilog='-------- End of the help message --------')

# Adding arguments
parser.add_argument ('--oce'     , help='oce model name', type=str, default='eORCA1.2',
                         choices=['ORCA2.3', 'eORCA1.2', 'eORCA1.4', 'eORCA1.4.2', 'eORCA025', 'eORCA025.1', 'eORCA025.4'] )
parser.add_argument ('--atm'     , help='atm model name (ICO* or LMD*)', type=str, default='ICO40'    )
parser.add_argument ('--type'    , help='Type of distribution', type=str, choices=['iceshelf', 'iceberg', 'nosouth', 'full'], default='full'  )
## parser.add_argument ('--dir'     , help='Directory of input file', type=str, default='./' )
parser.add_argument ('--repartition_file', help='Data files with iceberg melting' , type=str, default='./runoff-icb_DaiTrenberth_Depoorter_eORCA1_JD.nc' )
parser.add_argument ('--repartition_var' , help='Variable name for iceshelf'      , type=str, default=None)
parser.add_argument ('--grids' , help='grids file name', default='grids.nc' )
parser.add_argument ('--areas' , help='masks file name', default='areas.nc' )
parser.add_argument ('--masks' , help='areas file name', default='masks.nc' )
parser.add_argument ('--o2a'   , help='o2a file name'  , default='o2a.nc'   )
parser.add_argument ('--output'  , help='output rmp file name', default='rmp_tlmd_to_torc_calving.nc' )
parser.add_argument ('--fmt'     , help='NetCDF file format, using nco syntax', default='netcdf4',
                         choices=['classic', 'netcdf3', '64bit', '64bit_data', '64bit_data', 'netcdf4', 'netcdf4_classsic'] )
parser.add_argument ('--ocePerio', help='periodicity of ocean grid', type=float, default=0, choices=nemo.nperio_valid_range )

# Parse command line
myargs = parser.parse_args()

# Model Names
src_Name = myargs.atm ; dst_Name = myargs.oce
    
# Default vars
if myargs.repartition_var == None :
    # Runoff from Antarctica iceshelves (Depoorter, 2013)
    if myargs.type == 'iceshelf' : repartitionVar = 'sornfisf'
        
    # Runoff from Antarctica iceberg (Depoorter, 2013)
    if myargs.type == 'iceberg' :  repartitionVar = 'Icb_Flux'
        
else :                             repartitionVar = myargs.repartition_var

# Latitude limits of each calving zone
limit_lat = ( (40.0, 90.0), (-50.0, 40.0), ( -90.0, -50.0) )

nb_zone = len(limit_lat)

if myargs.fmt == 'classic'         : FmtNetcdf = 'CLASSIC'
if myargs.fmt == 'netcdf3'         : FmtNetcdf = 'CLASSIC'
if myargs.fmt == '64bit'           : FmtNetcdf = 'NETCDF3_64BIT_OFFSET'
if myargs.fmt == '64bit_data'      : FmtNetcdf = 'NETCDF3_64BIT_DATA'
if myargs.fmt == '64bit_offset'    : FmtNetcdf = 'NETCDF3_64BIT_OFFSET'
if myargs.fmt == 'netcdf4'         : FmtNetcdf = 'NETCDF4'
if myargs.fmt == 'netcdf4_classic' : FmtNetcdf = 'NETCDF4_CLASSIC'
### ==========================================================================================

# Model short names
src_name = None ; dst_name = None
if src_Name.count('ICO')  != 0 : src_name = 'ico'
if src_Name.count('LMD')  != 0 : src_name = 'lmd'
if dst_Name.count('ORCA') != 0 : dst_name = 'orc'

CplModel = dst_Name + 'x' + src_Name

print ('Atm names : ' + src_name + ' ' + src_Name )
print ('Oce names : ' + dst_name + ' ' + dst_Name )
print (' ')

# Reading domains characteristics
grids = myargs.grids
areas = myargs.areas
masks = myargs.masks
o2a   = myargs.o2a

print ('Opening ' + areas)
f_areas = xr.open_dataset ( areas )
print ('Opening ' + masks)
f_masks = xr.open_dataset ( masks )
print ('Opening ' + grids)
f_grids = xr.open_dataset ( grids )

src_lon = f_grids ['t' + src_name + '.lon']
src_lat = f_grids ['t' + src_name + '.lat']
dst_lon = f_grids ['t' + dst_name + '.lon']
dst_lat = f_grids ['t' + dst_name + '.lat']

src_msk = f_masks ['t' + src_name + '.msk']
dst_msk = f_masks ['t' + dst_name + '.msk']
dst_mskutil = 1-dst_msk # Reversing the convention : 0 on continent, 1 on ocean
print ('dst_msk     : ', dst_msk.sum().values )
print ('dst_mskutil : ', dst_mskutil.sum().values )

# Ocean grid periodicity
nperio_dst=myargs.ocePerio

# Periodicity masking for NEMO
if nperio_dst == 0 :
    if dst_Name in ['ORCA2.3', 'ORCA2.4']            : nperio_dst = 4
    if dst_Name in ['eORCA1.2', 'eORCA1.3']          : nperio_dst = 6
    if dst_Name in ['ORCA025', 'eORCA025', 'eORCA025.1', 'eORCA025.4'] : nperio_dst = 6
        
print ('nperio_dst: ' + str(nperio_dst) )
dst_mskutil = nemo.lbc_mask (dst_mskutil, nperio=nperio_dst, cd_type='T', sval=0 )
print ('dst_mskutil : ' + str(np.sum(dst_mskutil)))

##
## Fill Closed seas and other
## ==================================================

# Preparation by closing some straits
# -----------------------------------
if dst_Name in [ 'eORCA025', 'eORCA025.1', 'eORCA025.4' ] :
    print ('Filling some seas for ' + dst_Name )
    # Set Gibraltar strait to 0 to fill Mediterranean sea
    dst_mskutil[838:841,1125] = 0
    # Set Bal-El-Mandeb to 0 to fill Red Sea
    dst_mskutil[736,1321:1324] = 0
    # Set Stagerak to 0 to fill Baltic Sea
    dst_mskutil[961,1179:1182] = 0
    # Set Ormuz Strait to 0 to fill Arabo-Persian Gulf
    dst_mskutil[794:798,1374] = 0
    # Set Hudson Strait to 0 to fill Hudson Bay
    dst_mskutil[997:1012,907] = 0
    
if dst_Name in [ 'eORCA1.2', 'eORCA1.3', 'eORCA1.4', 'eORCA1.4.2' ] :
    print ('Filling some seas for ' + dst_Name)
    # Set Gibraltar strait to 0 to fill Mediterrean sea
    dst_mskutil[240, 283] = 0
    # Set Bal-El-Mandeb to 0 to fill Red Sea
    dst_mskutil[211:214, 332] = 0
    # Set Stagerak to 0 to fill Baltic Sea
    dst_mskutil[272:276, 293] = 0
    # Set Ormuz Strait to 0 to fill Arabo-Persian Gulf
    dst_mskutil[227:230, 345] = 0
    # Set Hudson Strait to 0 to fill Hudson Bay
    dst_mskutil[284,222:225] = 0
    
if dst_Name in [ 'ORCA2.3', 'ORCA2.4' ] :
    print ('Filling some seas for ' + dst_Name)
    # Set Gibraltar strait to 0 to fill Mediterrean sea
    dst_mskutil[101,139] = 0
    # Set Black Sea to zero. At the edge of the domain : binary_fill_holes fails
    dst_mskutil[ 99:111,   0:  5] = 0
    dst_mskutil[106:111, 173:182] = 0
    # Set Stagerak to 0 to fill Baltic Sea
    dst_mskutil[115,144] = 0
    # Set Hudson Strait to 0 to fill Hudson Bay
    dst_mskutil[120:123,110] = 0
    # Set Bal-El-Mandeb to 0 to fill Red Sea
    dst_mskutil[87:89,166] = 0

dst_closed = dst_mskutil

# Fill closed seas with image processing library
# ----------------------------------------------
dst_mskutil = nemo.lbc_mask ( 1-ndimage.binary_fill_holes (1-nemo.lbc(dst_mskutil, nperio=nperio_dst, cd_type='T')), nperio=nperio_dst, cd_type='T', sval=0)

# Surfaces
src_srf = f_areas.variables ['t' + src_name + '.srf']
dst_srf = f_areas.variables ['t' + dst_name + '.srf']
dst_srfutil = dst_srf * np.float64 (dst_mskutil)

dst_srfutil_sum = np.sum( dst_srfutil)

src_clo = f_grids ['t' + src_name + '.clo']
src_cla = f_grids ['t' + src_name + '.cla']
dst_clo = f_grids ['t' + dst_name + '.clo']
dst_cla = f_grids ['t' + dst_name + '.cla']

# Indices
( src_jpj, src_jpi) = src_lat.shape ; src_grid_size = src_jpj*src_jpi
( dst_jpj, dst_jpi) = dst_lat.shape ; dst_grid_size = dst_jpj*dst_jpi
orc_index = np.arange (dst_jpj*dst_jpi, dtype=np.int32) + 1 # Fortran indices (starting at one)

### ===== Reading needed data ==================================================
if myargs.type in ['iceberg', 'iceshelf' ]: 
    # Reading data file for calving or iceberg geometry around Antarctica
    print ( 'Opening ' + myargs.repartition_file)
    f_repartition = xr.open_dataset ( myargs.repartition_file )
    repartition = np.sum ( f_repartition.variables [repartitionVar], axis=0 )

## Before loop on basins
remap_matrix = np.empty ( shape=(0), dtype=np.float64 )
src_address  = np.empty ( shape=(0), dtype=np.int32   )
dst_address  = np.empty ( shape=(0), dtype=np.int32   )

print (' ')

### ===== Starting loop on basins==============================================

# Initialise some fields
remap_matrix = np.empty ( shape=(0), dtype=np.float64 )
src_address  = np.empty ( shape=(0), dtype=np.int32   )
dst_address  = np.empty ( shape=(0), dtype=np.int32   )

basin_msk       = np.zeros( shape=(nb_zone, dst_jpj, dst_jpi), dtype=np.float32)
key_repartition = np.zeros( shape=(nb_zone, dst_jpj, dst_jpi), dtype=np.float64)

## Loop
for n_bas in np.arange ( nb_zone ) :
    south = False ; ok_run = False
    lat_south = np.min(limit_lat[n_bas]) ; lat_north = np.max(limit_lat[n_bas])
    if lat_south <= -60.0 : south = True
    
    print ( 'basin: {:2d} -- Latitudes: {:+.1f} {:+.1f} --'.format(n_bas, lat_south, lat_north) )
    ##
    if myargs.type == 'iceberg'  and     south : ok_run = True  ; print ('Applying iceberg to south' )
    if myargs.type == 'iceshelf' and     south : ok_run = True  ; print ('Applying iceshelf to south' )
    if myargs.type == 'iceberg'  and not south : ok_run = False ; print ('Skipping iceberg: not south ')
    if myargs.type == 'iceshelf' and not south : ok_run = False ; print ('Skipping iceshelf: not south ')
    if myargs.type == 'nosouth'  and     south : ok_run = False ; print ('Skipping south: nosouth case' )
    if myargs.type == 'nosouth'  and not south : ok_run = True  ; print ('Running: not in south, uniform repartition')
    if myargs.type == 'full'                   : ok_run = True  ; print ('Running general trivial case, uniform repartition' )

    if ok_run :
        # Calving integral send to one point per latitude bands
        index_src = ((src_grid_size - 1)*n_bas) // (nb_zone-1) + 1
   
        # Basin mask
        basin_msk[n_bas] = np.where ( (dst_lat > lat_south ) & (dst_lat <= lat_north ), dst_mskutil, 0 )

        # Repartition pattern
        if myargs.type in ['iceberg', 'iceshelf' ] : key_repartition[n_bas] = repartition * basin_msk[n_bas]
        else :                                       key_repartition[n_bas] =               basin_msk[n_bas]

        # Integral and normalisation
        sum_repartition = np.sum ( key_repartition[n_bas] * dst_srfutil ).values
        key_repartition = key_repartition / sum_repartition
    
        print ( 'Sum of repartition key                      : {:12.3e}'.format (np.sum (key_repartition[n_bas]               )) )
        print ( 'Integral (area weighted) of repartition key : {:12.3e}'.format (np.sum (key_repartition[n_bas] * dst_srfutil ).values) )
    
        # Basin surface (modulated by repartition key)
        basin_srfutil = np.sum ( key_repartition[n_bas] * dst_srfutil )
            
        # Weights and links
        poids = 1.0 / basin_srfutil
        matrix_local   = np.where ( basin_msk[n_bas].ravel() > 0.5, key_repartition[n_bas].ravel()*poids.values, 0. )
        matrix_local   = matrix_local[matrix_local > 0.0] # Keep only non zero values
        num_links      = np.count_nonzero (matrix_local)
        # Address on source grid : all links points to the same LMDZ point.
        src_address_local = np.ones(num_links, dtype=np.int32 )*index_src
        # Address on destination grid : each NEMO point with non zero link
        dst_address_local = np.where ( key_repartition[n_bas].ravel() > 0.0, orc_index, 0)
        dst_address_local = dst_address_local[dst_address_local > 0]
        # Append to global tabs
        remap_matrix = np.append ( remap_matrix, matrix_local      )
        src_address  = np.append ( src_address , src_address_local )
        dst_address  = np.append ( dst_address , dst_address_local )
        #
        #print ( 'Sum of remap_matrix : {:12.3e}'.format(np.sum(matrix_local)) )
        print ( 'Point in atmospheric grid : {:4d} -- num_links: {:6d}'.format(index_src, num_links) ) 
        print ('  ')



## End of loop
print (' ')

# 
num_links = np.count_nonzero (remap_matrix)
print ( 'Total num_links : {:10d}'.format(num_links) )

# Diag : interpolate uniform field
src_field = np.zeros(shape=(src_grid_size))
for n_bas in np.arange ( nb_zone ) :
    index_src = ((src_grid_size - 1)*n_bas) // (nb_zone-1) + 1
    src_field[index_src-1] = n_bas

dst_field = np.zeros(shape=(dst_grid_size,))
for link in np.arange (num_links) :
    dst_field [dst_address [link]-1] +=  remap_matrix[link] * src_field[src_address[link]-1]

### ===== Writing the weights file, for OASIS MCT ==========================================

# Output file
calving = myargs.output
    
print ('Output file: ' + calving )

remap_matrix = xr.DataArray (np.reshape(remap_matrix, (num_links, 1)), dims = ['num_links', 'num_wgts'] )
src_address  = xr.DataArray (src_address.astype(np.int32) , dims = ['num_links',] )
src_address.attrs['convention'] = "Fortran style addressing, starting at 1"
dst_address  = xr.DataArray (dst_address.astype(np.int32) , dims =  ['num_links',] )
dst_address.attrs['convention'] = "Fortran style addressing, starting at 1"

src_grid_dims       = xr.DataArray ( np.array (( src_jpi, src_jpi ), dtype=np.int32), dims =  ['src_grid_rank',] )
src_grid_center_lon = xr.DataArray ( src_lon.values.ravel(), dims = ['src_grid_size',] )
src_grid_center_lat = xr.DataArray ( src_lat.values.ravel(), dims = ['src_grid_size',] )
src_grid_center_lon.attrs['units']='degrees_east'  ; src_grid_center_lon.attrs['long_name']='Longitude'
src_grid_center_lon.attrs['long_name']='longitude' ; src_grid_center_lon.attrs['bounds']="src_grid_corner_lon" 
src_grid_center_lat.attrs['units']='degrees_north' ; src_grid_center_lat.attrs['long_name']='Latitude'
src_grid_center_lat.attrs['long_name']='latitude ' ; src_grid_center_lat.attrs['bounds']="src_grid_corner_lat" 
src_grid_corner_lon = xr.DataArray ( src_clo.values.reshape( (src_jpi*src_jpj, src_clo.shape[0]) ), dims = ['src_grid_size', 'src_grid_corners'] )
src_grid_corner_lat = xr.DataArray ( src_cla.values.reshape( (src_jpi*src_jpj, src_clo.shape[0]) ), dims = ['src_grid_size', 'src_grid_corners' ] )
src_grid_corner_lon.attrs['units']="degrees_east"
src_grid_corner_lat.attrs['units']="degrees_north"
src_grid_area       = xr.DataArray ( src_srf.values.ravel(), dims = ['src_grid_size',] )
src_grid_area.attrs['long_name']="Grid area" ; src_grid_area.attrs['standard_name']="cell_area" ; src_grid_area.attrs['units']="m2"
src_grid_imask      = xr.DataArray ( src_msk.values.ravel().astype(np.int32) , dims = ['src_grid_size',] )
src_grid_imask.attrs['long_name']="Land-sea mask" ; src_grid_imask.attrs['units']="Land:1, Ocean:0"

# --
dst_grid_dims       = xr.DataArray ( np.array(( dst_jpi, dst_jpi), dtype=np.int32) , dims = ['dst_grid_rank', ])
dst_grid_center_lon = xr.DataArray ( dst_lon.values.ravel(), dims = ['dst_grid_size', ])
dst_grid_center_lat = xr.DataArray ( dst_lat.values.ravel(), dims = ['dst_grid_size', ])
dst_grid_center_lon.attrs['units']='degrees_east'  ; dst_grid_center_lon.attrs['long_name']='Longitude'
dst_grid_center_lon.attrs['long_name']='longitude' ; dst_grid_center_lon.attrs['bounds']="dst_grid_corner_lon" 
dst_grid_center_lat.attrs['units']='degrees_north' ; dst_grid_center_lat.attrs['long_name']='Latitude'
dst_grid_center_lat.attrs['long_name']='latitude'  ; dst_grid_center_lat.attrs['bounds']="dst_grid_corner_lat" 
dst_grid_corner_lon = xr.DataArray ( dst_clo.values.reshape(dst_jpi*dst_jpj, dst_clo.shape[0] ), dims = ['dst_grid_size', 'dst_grid_corners'] )
dst_grid_corner_lat = xr.DataArray ( dst_cla.values.reshape(dst_jpi*dst_jpj, dst_clo.shape[0] ), dims = ['dst_grid_size', 'dst_grid_corners'] )
dst_grid_corner_lon.attrs['units']="degrees_east"
dst_grid_corner_lat.attrs['units']="degrees_north"
dst_grid_area       = xr.DataArray ( dst_srf.values.ravel(), dims = ['dst_grid_size', ] )
dst_grid_area.attrs['long_name']="Grid area" ; dst_grid_area.attrs['standard_name']="cell_area" ; dst_grid_area.attrs['units']="m2"
dst_grid_imask      = xr.DataArray ( dst_msk.values.ravel(), dims =  ['dst_grid_size',]  )
dst_grid_imask.attrs['long_name']="Land-sea mask" ; dst_grid_imask.attrs['units']="Land:1, Ocean:0"

dst_bassin      = xr.DataArray ( basin_msk.reshape ( (nb_zone,dst_grid_size) ) , dims = ['nb_zone', 'dst_grid_size',  ] )
dst_repartition = xr.DataArray ( key_repartition.reshape( (nb_zone,dst_grid_size) ) , dims = ['nb_zone', 'dst_grid_size',  ] )

dst_southLimit = xr.DataArray ( np.min(limit_lat, axis=(1,)).astype(np.float32) , dims = ['nb_zone', ] )
dst_northLimit = xr.DataArray ( np.min(limit_lat, axis=(1,)).astype(np.float32) , dims = ['nb_zone', ] )

# Additionnal fields for debugging
# ================================
dst_grid_imaskutil  = xr.DataArray ( dst_mskutil.ravel().astype(np.float32), dims = ['dst_grid_size',] )
dst_grid_imaskutil.attrs['long_name']="Land-sea mask" ; dst_grid_imaskutil.attrs['units']="Land:1, Ocean:0"
dst_grid_imaskclose = xr.DataArray ( dst_closed.values.ravel(), dims = ['dst_grid_size',] )
dst_grid_imaskclose.attrs['long_name']="Land-sea mask" ; dst_grid_imaskclose.attrs['units']="Land:1, Ocean:0"

dst_lon_addressed = xr.DataArray ( dst_lon.values.ravel()[dst_address-1].ravel(), dims = ['num_links',] )
dst_lat_addressed = xr.DataArray ( dst_lat.values.ravel()[dst_address-1].ravel(), dims = ['num_links',] )
dst_lon_addressed.attrs['long_name']="Longitude" ; dst_lon_addressed.attrs['standard_name']="longitude" ; dst_lon_addressed.attrs['units']="degrees_east"
dst_lat_addressed.attrs['long_name']="Latitude"  ; dst_lat_addressed.attrs['standard_name']="latitude"  ; dst_lat_addressed.attrs['units']="degrees_north"
dst_area_addressed  = xr.DataArray ( dst_srf.values.ravel()[dst_address-1].ravel(), dims = ['num_links',] )
dst_area_addressed.attrs['long_name']="Grid area" ; dst_area_addressed.attrs['standard_name']="cell_area" ; dst_grid_area.attrs['units']="m2"
dst_imask_addressed = xr.DataArray ( dst_msk.values.ravel()[dst_address-1].ravel(), dims = ['num_links', ] )
dst_imask_addressed.attrs['long_name']="Land-sea mask" ; dst_imask_addressed.attrs['units']="Land:1, Ocean:0"
dst_imaskutil_addressed = xr.DataArray ( dst_mskutil.ravel()[dst_address-1].astype(np.float32), dims = ['num_links'] )
dst_imaskutil_addressed.attrs['long_name']="Land-sea mask" ; dst_imaskutil_addressed.attrs['units']="Land:1, Ocean:0"

src_field = xr.DataArray ( src_field, dims = ['src_grid_size',] )
dst_field = xr.DataArray ( dst_field, dims = ['dst_grid_size',] )

f_calving =  xr.Dataset ( { 
    'remap_matrix'            : remap_matrix,
	'src_address'             : src_address,
	'dst_address'             : dst_address,
	'src_grid_dims'           : src_grid_dims,
	'src_grid_center_lon'     : src_grid_center_lat,
	'src_grid_center_lat'     : src_grid_center_lat,
	'src_grid_corner_lon'     : src_grid_corner_lon,
	'src_grid_corner_lat'     : src_grid_corner_lat,
	'src_grid_area'           : src_grid_area,
	'src_grid_imask'          : src_grid_imask,
	'dst_grid_dims'           : dst_grid_dims,
	'dst_grid_center_lon'     : dst_grid_center_lon,
	'dst_grid_center_lat'     : dst_grid_center_lat,
	'dst_grid_corner_lon'     : dst_grid_corner_lon,
	'dst_grid_corner_lat'     : dst_grid_corner_lat,
	'dst_grid_area'           : dst_grid_area,
	'dst_grid_imask'          : dst_grid_imask,
	'dst_bassin'              : dst_bassin,
	'dst_repartition'         : dst_repartition,
	'dst_southLimit'          : dst_southLimit,
	'dst_northLimit'          : dst_northLimit,
	'dst_grid_imaskutil'      : dst_grid_imaskutil,
	'dst_grid_imaskclose'     : dst_grid_imaskclose,
	'dst_lon_addressed'       : dst_lon_addressed,
	'dst_lat_addressed'       : dst_lat_addressed,
	'dst_area_addressed'      : dst_area_addressed,
	'dst_imask_addressed'     : dst_imask_addressed,
	'dst_imaskutil_addressed' : dst_imaskutil_addressed,
	'src_field'               : src_field,
	'dst_field'               : dst_field,
    } )

f_calving.attrs['Conventions']     = "CF-1.6"
f_calving.attrs['source']          = "IPSL Earth system model"
f_calving.attrs['group']           = "ICMC IPSL Climate Modelling Center"
f_calving.attrs['Institution']     = "IPSL https.//www.ipsl.fr"
f_calving.attrs['Ocean']           = dst_Name + " https://www.nemo-ocean.eu"
f_calving.attrs['Atmosphere']      = src_Name + " http://lmdz.lmd.jussieu.fr"
if myargs.type in ['iceberg', 'iceshelf' ]: f_calving.attrs['originalFiles'] = myargs.repartition_file
f_calving.attrs['associatedFiles'] = grids + " " + areas + " " + masks

f_calving.attrs['description']     = "Generated with XIOS http://forge.ipsl.jussieu.fr/ioserver and MOSAIX https://forge.ipsl.jussieu.fr/igcmg/browser/TOOLS/MOSAIX"
f_calving.attrs['title']           = calving
f_calving.attrs['Program']         = "Generated by " + sys.argv[0] + " with flags " + ' '.join (sys.argv[1:]) 

f_calving.attrs['repartitionType'] = myargs.type
if myargs.type in [ 'iceberg', 'iceshelf' ] :
    f_calving.attrs['repartitionFile'] = myargs.repartition_file
    f_calving.attrs['repartitionVar']  = repartitionVar
f_calving.attrs['gridsFile']       = grids
f_calving.attrs['areasFile']       = areas
f_calving.attrs['masksFile']       = masks
f_calving.attrs['timeStamp']       = time.asctime()
try    : f_calving.attrs['directory'] = os.getcwd ()
except : pass
try    : f_runoff.attrs['HOSTNAME'] = platform.node ()
except : pass
try    : f_runoff.attrs['LOGNAME']  = os.getlogin ()
except : pass
try    : f_runoff.attrs['Python']   = "Python version " +  platform.python_version ()
except : pass
try    : f_runoff.attrs['OS']       = platform.system ()
except : pass
try    : f_runoff.attrs['release']  = platform.release ()
except : pass
try    : f_runoff.attrs['hardware'] = platform.machine ()
except : pass
f_calving.attrs['conventions']     = "SCRIP"
if src_name == 'lmd' : f_calving.attrs['source_grid'] = "curvilinear"
if src_name == 'ico' : f_calving.attrs['source_grid'] = "unstructured"
f_calving.attrs['dest_grid']       = "curvilinear"
f_calving.attrs['Model']           = "IPSL CM6"
f_calving.attrs['SVN_Author']      = "$Author$"
f_calving.attrs['SVN_Date']        = "$Date$"
f_calving.attrs['SVN_Revision']    = "$Revision$"
f_calving.attrs['SVN_Id']          = "$Id$"
f_calving.attrs['SVN_HeadURL']     = "$HeadURL$"

##
f_calving.to_netcdf ( calving, mode='w', format=FmtNetcdf )
f_calving.close()

print ('That''s all folks !')
## ======================================================================================
